@@ -8,29 +8,24 @@
 package hooks // import "golang.org/x/tools/gopls/internal/hooks"
 
 import (
-	"context"
-	"regexp"
-
-	"golang.org/x/tools/internal/lsp/source"
-	"mvdan.cc/gofumpt/format"
+	"golang.org/x/tools/gopls/internal/lsp/source"
+	"golang.org/x/tools/internal/diff"
 	"mvdan.cc/xurls/v2"
 )
 
 func Options(options *source.Options) {
 	options.LicensesText = licensesText
 	if options.GoDiff {
-		options.ComputeEdits = ComputeEdits
-	}
-	options.URLRegexp = urlRegexp()
-	options.GofumptFormat = func(ctx context.Context, src []byte) ([]byte, error) {
-		return format.Source(src, format.Options{})
+		switch options.NewDiff {
+		case "old":
+			options.ComputeEdits = ComputeEdits
+		case "new":
+			options.ComputeEdits = diff.Strings
+		default:
+			options.ComputeEdits = BothDiffs
+		}
 	}
+	options.URLRegexp = xurls.Relaxed()
 	updateAnalyzers(options)
-}
-
-func urlRegexp() *regexp.Regexp {
-	// Ensure links are matched as full words, not anywhere.
-	re := regexp.MustCompile(`\b(` + xurls.Relaxed().String() + `)\b`)
-	re.Longest()
-	return re
+	updateGofumpt(options)
 }
