@@ -8,9 +8,9 @@ import (
 	"strings"
 	"testing"
 
-	. "golang.org/x/tools/internal/lsp/regtest"
-
-	"golang.org/x/tools/internal/lsp/tests"
+	. "golang.org/x/tools/gopls/internal/lsp/regtest"
+	"golang.org/x/tools/gopls/internal/lsp/tests/compare"
+	"golang.org/x/tools/internal/testenv"
 )
 
 const unformattedProgram = `
@@ -34,10 +34,10 @@ func TestFormatting(t *testing.T) {
 	Run(t, unformattedProgram, func(t *testing.T, env *Env) {
 		env.OpenFile("main.go")
 		env.FormatBuffer("main.go")
-		got := env.Editor.BufferText("main.go")
+		got := env.BufferText("main.go")
 		want := env.ReadWorkspaceFile("main.go.golden")
 		if got != want {
-			t.Errorf("unexpected formatting result:\n%s", tests.Diff(t, want, got))
+			t.Errorf("unexpected formatting result:\n%s", compare.Text(want, got))
 		}
 	})
 }
@@ -56,10 +56,10 @@ func f() {}
 	Run(t, onelineProgram, func(t *testing.T, env *Env) {
 		env.OpenFile("a.go")
 		env.FormatBuffer("a.go")
-		got := env.Editor.BufferText("a.go")
+		got := env.BufferText("a.go")
 		want := env.ReadWorkspaceFile("a.go.formatted")
 		if got != want {
-			t.Errorf("unexpected formatting result:\n%s", tests.Diff(t, want, got))
+			t.Errorf("unexpected formatting result:\n%s", compare.Text(want, got))
 		}
 	})
 }
@@ -80,10 +80,10 @@ func f() { fmt.Println() }
 	Run(t, onelineProgramA, func(t *testing.T, env *Env) {
 		env.OpenFile("a.go")
 		env.OrganizeImports("a.go")
-		got := env.Editor.BufferText("a.go")
+		got := env.BufferText("a.go")
 		want := env.ReadWorkspaceFile("a.go.imported")
 		if got != want {
-			t.Errorf("unexpected formatting result:\n%s", tests.Diff(t, want, got))
+			t.Errorf("unexpected formatting result:\n%s", compare.Text(want, got))
 		}
 	})
 }
@@ -101,10 +101,10 @@ func f() {}
 	Run(t, onelineProgramB, func(t *testing.T, env *Env) {
 		env.OpenFile("a.go")
 		env.OrganizeImports("a.go")
-		got := env.Editor.BufferText("a.go")
+		got := env.BufferText("a.go")
 		want := env.ReadWorkspaceFile("a.go.imported")
 		if got != want {
-			t.Errorf("unexpected formatting result:\n%s", tests.Diff(t, want, got))
+			t.Errorf("unexpected formatting result:\n%s", compare.Text(want, got))
 		}
 	})
 }
@@ -147,10 +147,10 @@ func TestOrganizeImports(t *testing.T) {
 	Run(t, disorganizedProgram, func(t *testing.T, env *Env) {
 		env.OpenFile("main.go")
 		env.OrganizeImports("main.go")
-		got := env.Editor.BufferText("main.go")
+		got := env.BufferText("main.go")
 		want := env.ReadWorkspaceFile("main.go.organized")
 		if got != want {
-			t.Errorf("unexpected formatting result:\n%s", tests.Diff(t, want, got))
+			t.Errorf("unexpected formatting result:\n%s", compare.Text(want, got))
 		}
 	})
 }
@@ -159,10 +159,10 @@ func TestFormattingOnSave(t *testing.T) {
 	Run(t, disorganizedProgram, func(t *testing.T, env *Env) {
 		env.OpenFile("main.go")
 		env.SaveBuffer("main.go")
-		got := env.Editor.BufferText("main.go")
+		got := env.BufferText("main.go")
 		want := env.ReadWorkspaceFile("main.go.formatted")
 		if got != want {
-			t.Errorf("unexpected formatting result:\n%s", tests.Diff(t, want, got))
+			t.Errorf("unexpected formatting result:\n%s", compare.Text(want, got))
 		}
 	})
 }
@@ -259,10 +259,10 @@ func main() {
 				env.CreateBuffer("main.go", crlf)
 				env.Await(env.DoneWithOpen())
 				env.OrganizeImports("main.go")
-				got := env.Editor.BufferText("main.go")
+				got := env.BufferText("main.go")
 				got = strings.ReplaceAll(got, "\r\n", "\n") // convert everything to LF for simplicity
 				if tt.want != got {
-					t.Errorf("unexpected content after save:\n%s", tests.Diff(t, tt.want, got))
+					t.Errorf("unexpected content after save:\n%s", compare.Text(tt.want, got))
 				}
 			})
 		})
@@ -303,6 +303,7 @@ func main() {
 }
 
 func TestGofumptFormatting(t *testing.T) {
+	testenv.NeedsGo1Point(t, 18)
 
 	// Exercise some gofumpt formatting rules:
 	//  - No empty lines following an assignment operator
@@ -358,10 +359,10 @@ const Bar = 42
 	).Run(t, input, func(t *testing.T, env *Env) {
 		env.OpenFile("foo.go")
 		env.FormatBuffer("foo.go")
-		got := env.Editor.BufferText("foo.go")
+		got := env.BufferText("foo.go")
 		want := env.ReadWorkspaceFile("foo.go.formatted")
 		if got != want {
-			t.Errorf("unexpected formatting result:\n%s", tests.Diff(t, want, got))
+			t.Errorf("unexpected formatting result:\n%s", compare.Text(want, got))
 		}
 	})
 }
