@@ -7,16 +7,12 @@ package misc
 import (
 	"testing"
 
-	"golang.org/x/tools/internal/lsp/protocol"
-	. "golang.org/x/tools/internal/lsp/regtest"
-	"golang.org/x/tools/internal/lsp/source"
-	"golang.org/x/tools/internal/testenv"
+	"golang.org/x/tools/gopls/internal/lsp/protocol"
+	. "golang.org/x/tools/gopls/internal/lsp/regtest"
+	"golang.org/x/tools/gopls/internal/lsp/source"
 )
 
 func TestWorkspaceSymbolMissingMetadata(t *testing.T) {
-	// We get 2 symbols on 1.12, for some reason.
-	testenv.NeedsGo1Point(t, 13)
-
 	const files = `
 -- go.mod --
 module mod.com
@@ -26,13 +22,14 @@ go 1.17
 package p
 
 const C1 = "a.go"
--- ignore.go --
+-- exclude.go --
 
-// +build ignore
+//go:build exclude
+// +build exclude
 
-package ignore
+package exclude
 
-const C2 = "ignore.go"
+const C2 = "exclude.go"
 `
 
 	Run(t, files, func(t *testing.T, env *Env) {
@@ -44,7 +41,7 @@ const C2 = "ignore.go"
 
 		// Opening up an ignored file will result in an overlay with missing
 		// metadata, but this shouldn't break workspace symbols requests.
-		env.OpenFile("ignore.go")
+		env.OpenFile("exclude.go")
 		syms = env.WorkspaceSymbol("C")
 		if got, want := len(syms), 1; got != want {
 			t.Errorf("got %d symbols, want %d", got, want)
