@@ -11,11 +11,11 @@ import (
 	"testing"
 
 	"golang.org/x/tools/gopls/internal/hooks"
-	"golang.org/x/tools/internal/lsp/bug"
-	. "golang.org/x/tools/internal/lsp/regtest"
+	. "golang.org/x/tools/gopls/internal/lsp/regtest"
+	"golang.org/x/tools/gopls/internal/lsp/tests/compare"
+	"golang.org/x/tools/internal/bug"
 
-	"golang.org/x/tools/internal/lsp/protocol"
-	"golang.org/x/tools/internal/lsp/tests"
+	"golang.org/x/tools/gopls/internal/lsp/protocol"
 	"golang.org/x/tools/internal/testenv"
 )
 
@@ -67,8 +67,6 @@ const Name = "Hello"
 `
 
 func TestModFileModification(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
-
 	const untidyModule = `
 -- a/go.mod --
 module mod.com
@@ -95,10 +93,13 @@ func main() {
 			goModContent := env.ReadWorkspaceFile("a/go.mod")
 			env.OpenFile("a/main.go")
 			env.Await(
-				env.DiagnosticAtRegexp("a/main.go", "\"example.com/blah\""),
+				OnceMet(
+					env.DoneWithOpen(),
+					env.DiagnosticAtRegexp("a/main.go", "\"example.com/blah\""),
+				),
 			)
 			if got := env.ReadWorkspaceFile("a/go.mod"); got != goModContent {
-				t.Fatalf("go.mod changed on disk:\n%s", tests.Diff(t, goModContent, got))
+				t.Fatalf("go.mod changed on disk:\n%s", compare.Text(goModContent, got))
 			}
 			// Save the buffer, which will format and organize imports.
 			// Confirm that the go.mod file still does not change.
@@ -107,41 +108,57 @@ func main() {
 				env.DiagnosticAtRegexp("a/main.go", "\"example.com/blah\""),
 			)
 			if got := env.ReadWorkspaceFile("a/go.mod"); got != goModContent {
-				t.Fatalf("go.mod changed on disk:\n%s", tests.Diff(t, goModContent, got))
+				t.Fatalf("go.mod changed on disk:\n%s", compare.Text(goModContent, got))
 			}
 		})
 	})
 
 	// Reproduce golang/go#40269 by deleting and recreating main.go.
 	t.Run("delete main.go", func(t *testing.T) {
-		t.Skip("This test will be flaky until golang/go#40269 is resolved.")
-
 		runner.Run(t, untidyModule, func(t *testing.T, env *Env) {
 			goModContent := env.ReadWorkspaceFile("a/go.mod")
 			mainContent := env.ReadWorkspaceFile("a/main.go")
 			env.OpenFile("a/main.go")
 			env.SaveBuffer("a/main.go")
 
+			// Ensure that we're done processing all the changes caused by opening
+			// and saving above. If not, we may run into a file locking issue on
+			// windows.
+			//
+			// If this proves insufficient, env.RemoveWorkspaceFile can be updated to
+			// retry file lock errors on windows.
+			env.Await(
+				env.DoneWithOpen(),
+				env.DoneWithSave(),
+				env.DoneWithChangeWatchedFiles(),
+			)
 			env.RemoveWorkspaceFile("a/main.go")
+
+			// TODO(rfindley): awaiting here shouldn't really be necessary. We should
+			// be consistent eventually.
+			//
+			// Probably this was meant to exercise a race with the change below.
 			env.Await(
 				env.DoneWithOpen(),
 				env.DoneWithSave(),
 				env.DoneWithChangeWatchedFiles(),
 			)
 
-			env.WriteWorkspaceFile("main.go", mainContent)
+			env.WriteWorkspaceFile("a/main.go", mainContent)
 			env.Await(
-				env.DiagnosticAtRegexp("main.go", "\"example.com/blah\""),
+				OnceMet(
+					env.DoneWithChangeWatchedFiles(),
+					env.DiagnosticAtRegexp("a/main.go", "\"example.com/blah\""),
+				),
 			)
-			if got := env.ReadWorkspaceFile("go.mod"); got != goModContent {
-				t.Fatalf("go.mod changed on disk:\n%s", tests.Diff(t, goModContent, got))
+			if got := env.ReadWorkspaceFile("a/go.mod"); got != goModContent {
+				t.Fatalf("go.mod changed on disk:\n%s", compare.Text(goModContent, got))
 			}
 		})
 	})
 }
 
 func TestGoGetFix(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
 	const mod = `
 -- a/go.mod --
 module mod.com
@@ -186,14 +203,13 @@ require example.com v1.2.3
 		}
 		env.ApplyQuickFixes("a/main.go", []protocol.Diagnostic{goGetDiag})
 		if got := env.ReadWorkspaceFile("a/go.mod"); got != want {
-			t.Fatalf("unexpected go.mod content:\n%s", tests.Diff(t, want, got))
+			t.Fatalf("unexpected go.mod content:\n%s", compare.Text(want, got))
 		}
 	})
 }
 
 // Tests that multiple missing dependencies gives good single fixes.
 func TestMissingDependencyFixes(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
 	const mod = `
 -- a/go.mod --
 module mod.com
@@ -236,7 +252,7 @@ require random.org v1.2.3
 		}
 		env.ApplyQuickFixes("a/main.go", []protocol.Diagnostic{randomDiag})
 		if got := env.ReadWorkspaceFile("a/go.mod"); got != want {
-			t.Fatalf("unexpected go.mod content:\n%s", tests.Diff(t, want, got))
+			t.Fatalf("unexpected go.mod content:\n%s", compare.Text(want, got))
 		}
 	})
 }
@@ -292,14 +308,12 @@ require random.org v1.2.3
 		}
 		env.ApplyQuickFixes("a/main.go", []protocol.Diagnostic{randomDiag})
 		if got := env.ReadWorkspaceFile("a/go.mod"); got != want {
-			t.Fatalf("unexpected go.mod content:\n%s", tests.Diff(t, want, got))
+			t.Fatalf("unexpected go.mod content:\n%s", compare.Text(want, got))
 		}
 	})
 }
 
 func TestIndirectDependencyFix(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
-
 	const mod = `
 -- a/go.mod --
 module mod.com
@@ -338,14 +352,13 @@ require example.com v1.2.3
 			),
 		)
 		env.ApplyQuickFixes("a/go.mod", d.Diagnostics)
-		if got := env.Editor.BufferText("a/go.mod"); got != want {
-			t.Fatalf("unexpected go.mod content:\n%s", tests.Diff(t, want, got))
+		if got := env.BufferText("a/go.mod"); got != want {
+			t.Fatalf("unexpected go.mod content:\n%s", compare.Text(want, got))
 		}
 	})
 }
 
 func TestUnusedDiag(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
 
 	const proxy = `
 -- example.com@v1.0.0/x.go --
@@ -383,8 +396,8 @@ go 1.14
 			),
 		)
 		env.ApplyQuickFixes("a/go.mod", d.Diagnostics)
-		if got := env.Editor.BufferText("a/go.mod"); got != want {
-			t.Fatalf("unexpected go.mod content:\n%s", tests.Diff(t, want, got))
+		if got := env.BufferText("a/go.mod"); got != want {
+			t.Fatalf("unexpected go.mod content:\n%s", compare.Text(want, got))
 		}
 	})
 }
@@ -392,7 +405,6 @@ go 1.14
 // Test to reproduce golang/go#39041. It adds a new require to a go.mod file
 // that already has an unused require.
 func TestNewDepWithUnusedDep(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
 
 	const proxy = `
 -- github.com/esimov/caire@v1.2.5/go.mod --
@@ -456,7 +468,7 @@ require (
 )
 `
 		if got := env.ReadWorkspaceFile("a/go.mod"); got != want {
-			t.Fatalf("TestNewDepWithUnusedDep failed:\n%s", tests.Diff(t, want, got))
+			t.Fatalf("TestNewDepWithUnusedDep failed:\n%s", compare.Text(want, got))
 		}
 	})
 }
@@ -465,8 +477,6 @@ require (
 // the file watching GlobPattern in the capability registration. See
 // golang/go#39384.
 func TestModuleChangesOnDisk(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
-
 	const mod = `
 -- a/go.mod --
 module mod.com
@@ -498,8 +508,6 @@ func main() {
 // Tests golang/go#39784: a missing indirect dependency, necessary
 // due to blah@v2.0.0's incomplete go.mod file.
 func TestBadlyVersionedModule(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
-
 	const proxy = `
 -- example.com/blah/@v/v1.0.0.mod --
 module example.com
@@ -552,7 +560,7 @@ var _ = blah.Name
 			env.DiagnosticAtRegexpWithMessage("a/main.go", `"example.com/blah/v2"`, "cannot find module providing"),
 			env.DiagnosticAtRegexpWithMessage("a/go.mod", `require example.com/blah/v2`, "cannot find module providing"),
 		)
-		env.ApplyQuickFixes("a/go.mod", env.DiagnosticsFor("a/go.mod").Diagnostics)
+		env.ApplyQuickFixes("a/go.mod", env.Awaiter.DiagnosticsFor("a/go.mod").Diagnostics)
 		const want = `module mod.com
 
 go 1.12
@@ -564,8 +572,8 @@ require (
 `
 		env.SaveBuffer("a/go.mod")
 		env.Await(EmptyDiagnostics("a/main.go"))
-		if got := env.Editor.BufferText("a/go.mod"); got != want {
-			t.Fatalf("suggested fixes failed:\n%s", tests.Diff(t, want, got))
+		if got := env.BufferText("a/go.mod"); got != want {
+			t.Fatalf("suggested fixes failed:\n%s", compare.Text(want, got))
 		}
 	})
 }
@@ -575,9 +583,6 @@ func TestUnknownRevision(t *testing.T) {
 	if runtime.GOOS == "plan9" {
 		t.Skipf("skipping test that fails for unknown reasons on plan9; see https://go.dev/issue/50477")
 	}
-
-	testenv.NeedsGo1Point(t, 14)
-
 	const unknown = `
 -- a/go.mod --
 module mod.com
@@ -610,12 +615,10 @@ func main() {
 			env.SaveBuffer("a/go.mod") // Save to trigger diagnostics.
 
 			d := protocol.PublishDiagnosticsParams{}
-			env.Await(
-				OnceMet(
-					// Make sure the diagnostic mentions the new version -- the old diagnostic is in the same place.
-					env.DiagnosticAtRegexpWithMessage("a/go.mod", "example.com v1.2.3", "example.com@v1.2.3"),
-					ReadDiagnostics("a/go.mod", &d),
-				),
+			env.AfterChange(
+				// Make sure the diagnostic mentions the new version -- the old diagnostic is in the same place.
+				env.DiagnosticAtRegexpWithMessage("a/go.mod", "example.com v1.2.3", "example.com@v1.2.3"),
+				ReadDiagnostics("a/go.mod", &d),
 			)
 			qfs := env.GetQuickFixes("a/go.mod", d.Diagnostics)
 			if len(qfs) == 0 {
@@ -623,7 +626,7 @@ func main() {
 			}
 			env.ApplyCodeAction(qfs[0]) // Arbitrarily pick a single fix to apply. Applying all of them seems to cause trouble in this particular test.
 			env.SaveBuffer("a/go.mod")  // Save to trigger diagnostics.
-			env.Await(
+			env.AfterChange(
 				EmptyDiagnostics("a/go.mod"),
 				env.DiagnosticAtRegexp("a/main.go", "x = "),
 			)
@@ -654,17 +657,17 @@ func main() {
 	t.Run("good", func(t *testing.T) {
 		runner.Run(t, known, func(t *testing.T, env *Env) {
 			env.OpenFile("a/go.mod")
-			env.Await(
+			env.AfterChange(
 				env.DiagnosticAtRegexp("a/main.go", "x = "),
 			)
 			env.RegexpReplace("a/go.mod", "v1.2.3", "v1.2.2")
 			env.Editor.SaveBuffer(env.Ctx, "a/go.mod") // go.mod changes must be on disk
-			env.Await(
+			env.AfterChange(
 				env.DiagnosticAtRegexp("a/go.mod", "example.com v1.2.2"),
 			)
 			env.RegexpReplace("a/go.mod", "v1.2.2", "v1.2.3")
 			env.Editor.SaveBuffer(env.Ctx, "a/go.mod") // go.mod changes must be on disk
-			env.Await(
+			env.AfterChange(
 				env.DiagnosticAtRegexp("a/main.go", "x = "),
 			)
 		})
@@ -674,8 +677,6 @@ func main() {
 // Confirm that an error in an indirect dependency of a requirement is surfaced
 // as a diagnostic in the go.mod file.
 func TestErrorInIndirectDependency(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
-
 	const badProxy = `
 -- example.com@v1.2.3/go.mod --
 module example.com
@@ -740,13 +741,9 @@ func main() {
 }
 `
 	WithOptions(
-		EditorConfig{
-			Env: map[string]string{
-				"GOFLAGS": "-mod=readonly",
-			},
-		},
+		EnvVars{"GOFLAGS": "-mod=readonly"},
 		ProxyFiles(proxy),
-		Modes(Singleton),
+		Modes(Default),
 	).Run(t, mod, func(t *testing.T, env *Env) {
 		env.OpenFile("main.go")
 		original := env.ReadWorkspaceFile("go.mod")
@@ -755,7 +752,7 @@ func main() {
 		)
 		got := env.ReadWorkspaceFile("go.mod")
 		if got != original {
-			t.Fatalf("go.mod file modified:\n%s", tests.Diff(t, original, got))
+			t.Fatalf("go.mod file modified:\n%s", compare.Text(original, got))
 		}
 		env.RunGoCommand("get", "example.com/blah@v1.2.3")
 		env.RunGoCommand("mod", "tidy")
@@ -766,9 +763,15 @@ func main() {
 }
 
 func TestMultiModuleModDiagnostics(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
-
+	testenv.NeedsGo1Point(t, 18) // uses go.work
 	const mod = `
+-- go.work --
+go 1.18
+
+use (
+	a
+	b
+)
 -- a/go.mod --
 module moda.com
 
@@ -801,7 +804,6 @@ func main() {
 `
 	WithOptions(
 		ProxyFiles(workspaceProxy),
-		Modes(Experimental),
 	).Run(t, mod, func(t *testing.T, env *Env) {
 		env.Await(
 			env.DiagnosticAtRegexpWithMessage("a/go.mod", "example.com v1.2.3", "is not used"),
@@ -810,8 +812,6 @@ func main() {
 }
 
 func TestModTidyWithBuildTags(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
-
 	const mod = `
 -- go.mod --
 module mod.com
@@ -830,9 +830,7 @@ func main() {
 `
 	WithOptions(
 		ProxyFiles(workspaceProxy),
-		EditorConfig{
-			BuildFlags: []string{"-tags", "bob"},
-		},
+		Settings{"buildFlags": []string{"-tags", "bob"}},
 	).Run(t, mod, func(t *testing.T, env *Env) {
 		env.Await(
 			env.DiagnosticAtRegexp("main.go", `"example.com/blah"`),
@@ -861,10 +859,6 @@ func main() {}
 }
 
 func TestSumUpdateFixesDiagnostics(t *testing.T) {
-	t.Skipf("Skipping known-flaky test; see https://go.dev/issue/51352.")
-
-	testenv.NeedsGo1Point(t, 14)
-
 	const mod = `
 -- go.mod --
 module mod.com
@@ -928,7 +922,7 @@ func hello() {}
 		// TODO(rFindley) this doesn't work in multi-module workspace mode, because
 		// it keeps around the last parsing modfile. Update this test to also
 		// exercise the workspace module.
-		Modes(Singleton),
+		Modes(Default),
 	).Run(t, mod, func(t *testing.T, env *Env) {
 		env.OpenFile("go.mod")
 		env.Await(env.DoneWithOpen())
@@ -949,8 +943,6 @@ func hello() {}
 }
 
 func TestRemoveUnusedDependency(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
-
 	const proxy = `
 -- hasdep.com@v1.2.3/go.mod --
 module hasdep.com
@@ -1011,8 +1003,8 @@ func main() {}
 go 1.12
 `
 			env.ApplyQuickFixes("go.mod", d.Diagnostics)
-			if got := env.Editor.BufferText("go.mod"); got != want {
-				t.Fatalf("unexpected content in go.mod:\n%s", tests.Diff(t, want, got))
+			if got := env.BufferText("go.mod"); got != want {
+				t.Fatalf("unexpected content in go.mod:\n%s", compare.Text(want, got))
 			}
 		})
 	})
@@ -1064,15 +1056,14 @@ require random.com v1.2.3
 				diagnostics = append(diagnostics, d)
 			}
 			env.ApplyQuickFixes("go.mod", diagnostics)
-			if got := env.Editor.BufferText("go.mod"); got != want {
-				t.Fatalf("unexpected content in go.mod:\n%s", tests.Diff(t, want, got))
+			if got := env.BufferText("go.mod"); got != want {
+				t.Fatalf("unexpected content in go.mod:\n%s", compare.Text(want, got))
 			}
 		})
 	})
 }
 
 func TestSumUpdateQuickFix(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
 	const mod = `
 -- go.mod --
 module mod.com
@@ -1096,7 +1087,7 @@ func main() {
 `
 	WithOptions(
 		ProxyFiles(workspaceProxy),
-		Modes(Singleton),
+		Modes(Default),
 	).Run(t, mod, func(t *testing.T, env *Env) {
 		env.OpenFile("go.mod")
 		params := &protocol.PublishDiagnosticsParams{}
@@ -1111,14 +1102,12 @@ func main() {
 example.com v1.2.3/go.mod h1:Y2Rc5rVWjWur0h3pd9aEvK5Pof8YKDANh9gHA2Maujo=
 `
 		if got := env.ReadWorkspaceFile("go.sum"); got != want {
-			t.Fatalf("unexpected go.sum contents:\n%s", tests.Diff(t, want, got))
+			t.Fatalf("unexpected go.sum contents:\n%s", compare.Text(want, got))
 		}
 	})
 }
 
 func TestDownloadDeps(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14)
-
 	const proxy = `
 -- example.com@v1.2.3/go.mod --
 module example.com
@@ -1165,7 +1154,7 @@ func main() {
 `
 	WithOptions(
 		ProxyFiles(proxy),
-		Modes(Singleton),
+		Modes(Default),
 	).Run(t, mod, func(t *testing.T, env *Env) {
 		env.OpenFile("main.go")
 		d := &protocol.PublishDiagnosticsParams{}
@@ -1175,14 +1164,13 @@ func main() {
 		)
 		env.ApplyQuickFixes("main.go", d.Diagnostics)
 		env.Await(
-			EmptyDiagnostics("main.go"),
-			NoDiagnostics("go.mod"),
+			EmptyOrNoDiagnostics("main.go"),
+			EmptyOrNoDiagnostics("go.mod"),
 		)
 	})
 }
 
 func TestInvalidGoVersion(t *testing.T) {
-	testenv.NeedsGo1Point(t, 14) // Times out on 1.13 for reasons unclear. Not worth worrying about.
 	const files = `
 -- go.mod --
 module mod.com
