@@ -5,19 +5,21 @@
 package workspace
 
 import (
+	"context"
 	"fmt"
 	"path/filepath"
-	"sort"
 	"strings"
 	"testing"
 
 	"golang.org/x/tools/gopls/internal/hooks"
-	"golang.org/x/tools/internal/lsp/bug"
-	"golang.org/x/tools/internal/lsp/fake"
-	"golang.org/x/tools/internal/lsp/protocol"
+	"golang.org/x/tools/gopls/internal/lsp"
+	"golang.org/x/tools/gopls/internal/lsp/fake"
+	"golang.org/x/tools/gopls/internal/lsp/protocol"
+	"golang.org/x/tools/internal/bug"
+	"golang.org/x/tools/internal/gocommand"
 	"golang.org/x/tools/internal/testenv"
 
-	. "golang.org/x/tools/internal/lsp/regtest"
+	. "golang.org/x/tools/gopls/internal/lsp/regtest"
 )
 
 func TestMain(m *testing.M) {
@@ -33,6 +35,8 @@ go 1.12
 -- example.com@v1.2.3/blah/blah.go --
 package blah
 
+import "fmt"
+
 func SaySomething() {
 	fmt.Println("something")
 }
@@ -60,7 +64,7 @@ require (
 	random.org v1.2.3
 )
 -- pkg/go.sum --
-example.com v1.2.3 h1:Yryq11hF02fEf2JlOS2eph+ICE2/ceevGV3C9dl5V/c=
+example.com v1.2.3 h1:veRD4tUnatQRgsULqULZPjeoBGFr2qBhevSCZllD2Ds=
 example.com v1.2.3/go.mod h1:Y2Rc5rVWjWur0h3pd9aEvK5Pof8YKDANh9gHA2Maujo=
 random.org v1.2.3 h1:+JE2Fkp7gS0zsHXGEQJ7hraom3pNTlkxC4b2qPfA+/Q=
 random.org v1.2.3/go.mod h1:E9KM6+bBX2g5ykHZ9H27w16sWo3QwgonyjM44Dnej3I=
@@ -137,24 +141,6 @@ func TestReferences(t *testing.T) {
 	}
 }
 
-func TestDirectoryFilters(t *testing.T) {
-	WithOptions(
-		ProxyFiles(workspaceProxy),
-		WorkspaceFolders("pkg"),
-		Settings{
-			"directoryFilters": []string{"-inner"},
-		},
-	).Run(t, workspaceModule, func(t *testing.T, env *Env) {
-		syms := env.WorkspaceSymbol("Hi")
-		sort.Slice(syms, func(i, j int) bool { return syms[i].ContainerName < syms[j].ContainerName })
-		for _, s := range syms {
-			if strings.Contains(s.ContainerName, "inner") {
-				t.Errorf("WorkspaceSymbol: found symbol %q with container %q, want \"inner\" excluded", s.Name, s.ContainerName)
-			}
-		}
-	})
-}
-
 // Make sure that analysis diagnostics are cleared for the whole package when
 // the only opened file is closed. This test was inspired by the experience in
 // VS Code, where clicking on a reference result triggers a
@@ -175,6 +161,31 @@ func TestClearAnalysisDiagnostics(t *testing.T) {
 	})
 }
 
+// TestReloadOnlyOnce checks that changes to the go.mod file do not result in
+// redundant package loads (golang/go#54473).
+//
+// Note that this test may be fragile, as it depends on specific structure to
+// log messages around reinitialization. Nevertheless, it is important for
+// guarding against accidentally duplicate reloading.
+func TestReloadOnlyOnce(t *testing.T) {
+	WithOptions(
+		ProxyFiles(workspaceProxy),
+		WorkspaceFolders("pkg"),
+	).Run(t, workspaceModule, func(t *testing.T, env *Env) {
+		dir := env.Sandbox.Workdir.URI("goodbye").SpanURI().Filename()
+		goModWithReplace := fmt.Sprintf(`%s
+replace random.org => %s
+`, env.ReadWorkspaceFile("pkg/go.mod"), dir)
+		env.WriteWorkspaceFile("pkg/go.mod", goModWithReplace)
+		env.Await(
+			OnceMet(
+				env.DoneWithChangeWatchedFiles(),
+				LogMatching(protocol.Info, `packages\.Load #\d+\n`, 2, false),
+			),
+		)
+	})
+}
+
 // This test checks that gopls updates the set of files it watches when a
 // replace target is added to the go.mod.
 func TestWatchReplaceTargets(t *testing.T) {
@@ -207,6 +218,8 @@ go 1.12
 -- example.com@v1.2.3/blah/blah.go --
 package blah
 
+import "fmt"
+
 func SaySomething() {
 	fmt.Println("something")
 }
@@ -221,6 +234,7 @@ func Hello() {}
 `
 
 func TestAutomaticWorkspaceModule_Interdependent(t *testing.T) {
+	testenv.NeedsGo1Point(t, 18) // uses go.work
 	const multiModule = `
 -- moda/a/go.mod --
 module a.com
@@ -252,9 +266,10 @@ func Hello() int {
 `
 	WithOptions(
 		ProxyFiles(workspaceModuleProxy),
-		Modes(Experimental),
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
-		env.Await(
+		env.RunGoCommand("work", "init")
+		env.RunGoCommand("work", "use", "-r", ".")
+		env.AfterChange(
 			env.DiagnosticAtRegexp("moda/a/a.go", "x"),
 			env.DiagnosticAtRegexp("modb/b/b.go", "x"),
 			env.NoDiagnosticAtRegexp("moda/a/a.go", `"b.com/b"`),
@@ -262,9 +277,7 @@ func Hello() int {
 	})
 }
 
-func TestMultiModuleWithExclude(t *testing.T) {
-	testenv.NeedsGo1Point(t, 16)
-
+func TestModuleWithExclude(t *testing.T) {
 	const proxy = `
 -- c.com@v1.2.3/go.mod --
 module c.com
@@ -275,6 +288,8 @@ require b.com v1.2.3
 -- c.com@v1.2.3/blah/blah.go --
 package blah
 
+import "fmt"
+
 func SaySomething() {
 	fmt.Println("something")
 }
@@ -290,10 +305,6 @@ func Hello() {}
 module b.com
 
 go 1.12
--- b.com@v1.2.4/b/b.go --
-package b
-
-func Hello() {}
 `
 	const multiModule = `
 -- go.mod --
@@ -314,7 +325,6 @@ func main() {
 `
 	WithOptions(
 		ProxyFiles(proxy),
-		Modes(Experimental),
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
 		env.Await(
 			env.DiagnosticAtRegexp("main.go", "x"),
@@ -324,10 +334,19 @@ func main() {
 
 // This change tests that the version of the module used changes after it has
 // been deleted from the workspace.
+//
+// TODO(golang/go#55331): delete this placeholder along with experimental
+// workspace module.
 func TestDeleteModule_Interdependent(t *testing.T) {
-	t.Skip("Skipping due to golang/go#46375: race due to orphaned file reloading")
-
+	testenv.NeedsGo1Point(t, 18) // uses go.work
 	const multiModule = `
+-- go.work --
+go 1.18
+
+use (
+	moda/a
+	modb
+)
 -- moda/a/go.mod --
 module a.com
 
@@ -358,7 +377,6 @@ func Hello() int {
 `
 	WithOptions(
 		ProxyFiles(workspaceModuleProxy),
-		Modes(Experimental),
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
 		env.OpenFile("moda/a/a.go")
 		env.Await(env.DoneWithOpen())
@@ -368,23 +386,13 @@ func Hello() int {
 			t.Errorf("expected %s, got %v", want, original)
 		}
 		env.CloseBuffer(original)
-		env.Await(env.DoneWithClose())
+		env.AfterChange()
 
 		env.RemoveWorkspaceFile("modb/b/b.go")
 		env.RemoveWorkspaceFile("modb/go.mod")
-		env.Await(
-			env.DoneWithChangeWatchedFiles(),
-		)
+		env.WriteWorkspaceFile("go.work", "go 1.18\nuse moda/a")
+		env.AfterChange()
 
-		d := protocol.PublishDiagnosticsParams{}
-		env.Await(
-			OnceMet(
-				env.DiagnosticAtRegexpWithMessage("moda/a/go.mod", "require b.com v1.2.3", "b.com@v1.2.3 has not been downloaded"),
-				ReadDiagnostics("moda/a/go.mod", &d),
-			),
-		)
-		env.ApplyQuickFixes("moda/a/go.mod", d.Diagnostics)
-		env.Await(env.DoneWithChangeWatchedFiles())
 		got, _ := env.GoToDefinition("moda/a/a.go", env.RegexpSearch("moda/a/a.go", "Hello"))
 		if want := "b.com@v1.2.3/b/b.go"; !strings.HasSuffix(got, want) {
 			t.Errorf("expected %s, got %v", want, got)
@@ -395,7 +403,14 @@ func Hello() int {
 // Tests that the version of the module used changes after it has been added
 // to the workspace.
 func TestCreateModule_Interdependent(t *testing.T) {
+	testenv.NeedsGo1Point(t, 18) // uses go.work
 	const multiModule = `
+-- go.work --
+go 1.18
+
+use (
+	moda/a
+)
 -- moda/a/go.mod --
 module a.com
 
@@ -416,7 +431,6 @@ func main() {
 }
 `
 	WithOptions(
-		Modes(Experimental),
 		ProxyFiles(workspaceModuleProxy),
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
 		env.OpenFile("moda/a/a.go")
@@ -426,6 +440,13 @@ func main() {
 		}
 		env.CloseBuffer(original)
 		env.WriteWorkspaceFiles(map[string]string{
+			"go.work": `go 1.18
+
+use (
+	moda/a
+	modb
+)
+`,
 			"modb/go.mod": "module b.com",
 			"modb/b/b.go": `package b
 
@@ -434,12 +455,7 @@ func Hello() int {
 }
 `,
 		})
-		env.Await(
-			OnceMet(
-				env.DoneWithChangeWatchedFiles(),
-				env.DiagnosticAtRegexp("modb/b/b.go", "x"),
-			),
-		)
+		env.AfterChange(env.DiagnosticAtRegexp("modb/b/b.go", "x"))
 		got, _ := env.GoToDefinition("moda/a/a.go", env.RegexpSearch("moda/a/a.go", "Hello"))
 		if want := "modb/b/b.go"; !strings.HasSuffix(got, want) {
 			t.Errorf("expected %s, got %v", want, original)
@@ -450,7 +466,17 @@ func Hello() int {
 // This test confirms that a gopls workspace can recover from initialization
 // with one invalid module.
 func TestOneBrokenModule(t *testing.T) {
+	t.Skip("golang/go#55331: this test is temporarily broken as go.work handling tries to build the workspace module")
+
+	testenv.NeedsGo1Point(t, 18) // uses go.work
 	const multiModule = `
+-- go.work --
+go 1.18
+
+use (
+	moda/a
+	modb
+)
 -- moda/a/go.mod --
 module a.com
 
@@ -479,7 +505,6 @@ func Hello() int {
 `
 	WithOptions(
 		ProxyFiles(workspaceModuleProxy),
-		Modes(Experimental),
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
 		env.OpenFile("modb/go.mod")
 		env.Await(
@@ -499,7 +524,6 @@ func Hello() int {
 func TestUseGoplsMod(t *testing.T) {
 	// This test validates certain functionality related to using a gopls.mod
 	// file to specify workspace modules.
-	testenv.NeedsGo1Point(t, 14)
 	const multiModule = `
 -- moda/a/go.mod --
 module a.com
@@ -524,7 +548,7 @@ module b.com
 
 require example.com v1.2.3
 -- modb/go.sum --
-example.com v1.2.3 h1:Yryq11hF02fEf2JlOS2eph+ICE2/ceevGV3C9dl5V/c=
+example.com v1.2.3 h1:veRD4tUnatQRgsULqULZPjeoBGFr2qBhevSCZllD2Ds=
 example.com v1.2.3/go.mod h1:Y2Rc5rVWjWur0h3pd9aEvK5Pof8YKDANh9gHA2Maujo=
 -- modb/b/b.go --
 package b
@@ -580,21 +604,16 @@ require (
 replace a.com => %s/moda/a
 replace b.com => %s/modb
 `, workdir, workdir))
-		env.Await(env.DoneWithChangeWatchedFiles())
-		// Check that go.mod diagnostics picked up the newly active mod file.
-		// The local version of modb has an extra dependency we need to download.
-		env.OpenFile("modb/go.mod")
-		env.Await(env.DoneWithOpen())
 
-		var d protocol.PublishDiagnosticsParams
+		// As of golang/go#54069, writing a gopls.mod to the workspace triggers a
+		// workspace reload.
 		env.Await(
 			OnceMet(
-				env.DiagnosticAtRegexpWithMessage("modb/go.mod", `require example.com v1.2.3`, "has not been downloaded"),
-				ReadDiagnostics("modb/go.mod", &d),
+				env.DoneWithChangeWatchedFiles(),
+				env.DiagnosticAtRegexp("modb/b/b.go", "x"),
 			),
 		)
-		env.ApplyQuickFixes("modb/go.mod", d.Diagnostics)
-		env.Await(env.DiagnosticAtRegexp("modb/b/b.go", "x"))
+
 		// Jumping to definition should now go to b.com in the workspace.
 		if err := checkHelloLocation("modb/b/b.go"); err != nil {
 			t.Fatal(err)
@@ -647,7 +666,6 @@ module example.com/bar
 func TestUseGoWork(t *testing.T) {
 	// This test validates certain functionality related to using a go.work
 	// file to specify workspace modules.
-	testenv.NeedsGo1Point(t, 14)
 	const multiModule = `
 -- moda/a/go.mod --
 module a.com
@@ -721,21 +739,15 @@ use (
 	./modb
 )
 `)
-		env.Await(env.DoneWithChangeWatchedFiles())
-		// Check that go.mod diagnostics picked up the newly active mod file.
-		// The local version of modb has an extra dependency we need to download.
-		env.OpenFile("modb/go.mod")
-		env.Await(env.DoneWithOpen())
 
-		var d protocol.PublishDiagnosticsParams
+		// As of golang/go#54069, writing go.work to the workspace triggers a
+		// workspace reload.
 		env.Await(
 			OnceMet(
-				env.DiagnosticAtRegexpWithMessage("modb/go.mod", `require example.com v1.2.3`, "has not been downloaded"),
-				ReadDiagnostics("modb/go.mod", &d),
+				env.DoneWithChangeWatchedFiles(),
+				env.DiagnosticAtRegexp("modb/b/b.go", "x"),
 			),
 		)
-		env.ApplyQuickFixes("modb/go.mod", d.Diagnostics)
-		env.Await(env.DiagnosticAtRegexp("modb/b/b.go", "x"))
 
 		// Jumping to definition should now go to b.com in the workspace.
 		if err := checkHelloLocation("modb/b/b.go"); err != nil {
@@ -930,8 +942,6 @@ use (
 }
 
 func TestNonWorkspaceFileCreation(t *testing.T) {
-	testenv.NeedsGo1Point(t, 13)
-
 	const files = `
 -- go.mod --
 module mod.com
@@ -953,8 +963,32 @@ var _ = fmt.Printf
 	})
 }
 
-func TestMultiModuleV2(t *testing.T) {
+func TestGoWork_V2Module(t *testing.T) {
+	testenv.NeedsGo1Point(t, 18) // uses go.work
+	// When using a go.work, we must have proxy content even if it is replaced.
+	const proxy = `
+-- b.com/v2@v2.1.9/go.mod --
+module b.com/v2
+
+go 1.12
+-- b.com/v2@v2.1.9/b/b.go --
+package b
+
+func Ciao()() int {
+	return 0
+}
+`
+
 	const multiModule = `
+-- go.work --
+go 1.18
+
+use (
+	moda/a
+	modb
+	modb/v2
+	modc
+)
 -- moda/a/go.mod --
 module a.com
 
@@ -997,122 +1031,42 @@ func main() {
 	var x int
 }
 `
+
 	WithOptions(
-		Modes(Experimental),
+		ProxyFiles(proxy),
 	).Run(t, multiModule, func(t *testing.T, env *Env) {
 		env.Await(
-			env.DiagnosticAtRegexp("moda/a/a.go", "x"),
-			env.DiagnosticAtRegexp("modb/b/b.go", "x"),
-			env.DiagnosticAtRegexp("modb/v2/b/b.go", "x"),
-			env.DiagnosticAtRegexp("modc/main.go", "x"),
+			OnceMet(
+				InitialWorkspaceLoad,
+				// TODO(rfindley): assert on the full set of diagnostics here. We
+				// should ensure that we don't have a diagnostic at b.Hi in a.go.
+				env.DiagnosticAtRegexp("moda/a/a.go", "x"),
+				env.DiagnosticAtRegexp("modb/b/b.go", "x"),
+				env.DiagnosticAtRegexp("modb/v2/b/b.go", "x"),
+				env.DiagnosticAtRegexp("modc/main.go", "x"),
+			),
 		)
 	})
 }
 
-func TestDirectoryFiltersLoads(t *testing.T) {
-	// exclude, and its error, should be excluded from the workspace.
-	const files = `
--- go.mod --
-module example.com
-
-go 1.12
--- exclude/exclude.go --
-package exclude
-
-const _ = Nonexistant
-`
-
-	WithOptions(
-		Settings{"directoryFilters": []string{"-exclude"}},
-	).Run(t, files, func(t *testing.T, env *Env) {
-		env.Await(NoDiagnostics("exclude/x.go"))
-	})
-}
-
-func TestDirectoryFiltersTransitiveDep(t *testing.T) {
-	// Even though exclude is excluded from the workspace, it should
-	// still be importable as a non-workspace package.
-	const files = `
--- go.mod --
-module example.com
-
-go 1.12
--- include/include.go --
-package include
-import "example.com/exclude"
-
-const _ = exclude.X
--- exclude/exclude.go --
-package exclude
-
-const _ = Nonexistant // should be ignored, since this is a non-workspace package
-const X = 1
-`
-
-	WithOptions(
-		Settings{"directoryFilters": []string{"-exclude"}},
-	).Run(t, files, func(t *testing.T, env *Env) {
-		env.Await(
-			NoDiagnostics("exclude/exclude.go"), // filtered out
-			NoDiagnostics("include/include.go"), // successfully builds
-		)
-	})
-}
-
-func TestDirectoryFiltersWorkspaceModules(t *testing.T) {
-	// Define a module include.com which should be in the workspace, plus a
-	// module exclude.com which should be excluded and therefore come from
-	// the proxy.
-	const files = `
--- include/go.mod --
-module include.com
-
-go 1.12
-
-require exclude.com v1.0.0
-
--- include/go.sum --
-exclude.com v1.0.0 h1:Q5QSfDXY5qyNCBeUiWovUGqcLCRZKoTs9XdBeVz+w1I=
-exclude.com v1.0.0/go.mod h1:hFox2uDlNB2s2Jfd9tHlQVfgqUiLVTmh6ZKat4cvnj4=
-
--- include/include.go --
-package include
-
-import "exclude.com"
-
-var _ = exclude.X // satisfied only by the workspace version
--- exclude/go.mod --
-module exclude.com
-
-go 1.12
--- exclude/exclude.go --
-package exclude
-
-const X = 1
-`
-	const proxy = `
--- exclude.com@v1.0.0/go.mod --
-module exclude.com
-
-go 1.12
--- exclude.com@v1.0.0/exclude.go --
-package exclude
-`
-	WithOptions(
-		Modes(Experimental),
-		ProxyFiles(proxy),
-		Settings{"directoryFilters": []string{"-exclude"}},
-	).Run(t, files, func(t *testing.T, env *Env) {
-		env.Await(env.DiagnosticAtRegexp("include/include.go", `exclude.(X)`))
-	})
-}
-
 // Confirm that a fix for a tidy module will correct all modules in the
 // workspace.
 func TestMultiModule_OneBrokenModule(t *testing.T) {
-	testenv.NeedsGo1Point(t, 15)
+	// In the earlier 'experimental workspace mode', gopls would aggregate go.sum
+	// entries for the workspace module, allowing it to correctly associate
+	// missing go.sum with diagnostics. With go.work files, this doesn't work:
+	// the go.command will happily write go.work.sum.
+	t.Skip("golang/go#57509: go.mod diagnostics do not work in go.work mode")
+	testenv.NeedsGo1Point(t, 18) // uses go.work
+	const files = `
+-- go.work --
+go 1.18
 
-	const mod = `
+use (
+	a
+	b
+)
+-- go.work.sum --
 -- a/go.mod --
 module a.com
 
@@ -1139,8 +1093,7 @@ func main() {
 `
 	WithOptions(
 		ProxyFiles(workspaceProxy),
-		Modes(Experimental),
-	).Run(t, mod, func(t *testing.T, env *Env) {
+	).Run(t, files, func(t *testing.T, env *Env) {
 		params := &protocol.PublishDiagnosticsParams{}
 		env.OpenFile("b/go.mod")
 		env.Await(
@@ -1201,13 +1154,17 @@ package main
 	})
 }
 
-func TestAddGoWork(t *testing.T) {
+func TestAddAndRemoveGoWork(t *testing.T) {
+	// Use a workspace with a module in the root directory to exercise the case
+	// where a go.work is added to the existing root directory. This verifies
+	// that we're detecting changes to the module source, not just the root
+	// directory.
 	const nomod = `
--- a/go.mod --
+-- go.mod --
 module a.com
 
 go 1.16
--- a/main.go --
+-- main.go --
 package main
 
 func main() {}
@@ -1223,29 +1180,45 @@ func main() {}
 	WithOptions(
 		Modes(Default),
 	).Run(t, nomod, func(t *testing.T, env *Env) {
-		env.OpenFile("a/main.go")
+		env.OpenFile("main.go")
 		env.OpenFile("b/main.go")
-		env.Await(
-			DiagnosticAt("a/main.go", 0, 0),
+		// Since b/main.go is not in the workspace, it should have a warning on its
+		// package declaration.
+		env.AfterChange(
+			EmptyDiagnostics("main.go"),
 			DiagnosticAt("b/main.go", 0, 0),
 		)
 		env.WriteWorkspaceFile("go.work", `go 1.16
 
 use (
-	a
+	.
 	b
 )
 `)
-		env.Await(NoOutstandingDiagnostics())
+		env.AfterChange(NoOutstandingDiagnostics())
+		// Removing the go.work file should put us back where we started.
+		env.RemoveWorkspaceFile("go.work")
+
+		// TODO(golang/go#57558, golang/go#57508): file watching is asynchronous,
+		// and we must wait for the view to be reconstructed before touching
+		// b/main.go, so that the new view "knows" about b/main.go. This is simply
+		// a bug, but awaiting the change here avoids it.
+		env.Await(env.DoneWithChangeWatchedFiles())
+
+		// TODO(rfindley): fix this bug: reopening b/main.go is necessary here
+		// because we no longer "see" the file in any view.
+		env.CloseBuffer("b/main.go")
+		env.OpenFile("b/main.go")
+
+		env.AfterChange(
+			EmptyDiagnostics("main.go"),
+			DiagnosticAt("b/main.go", 0, 0),
+		)
 	})
 }
 
 // Tests the fix for golang/go#52500.
 func TestChangeTestVariant_Issue52500(t *testing.T) {
-	// This test fails for unknown reasons at Go <= 15. Presumably the loading of
-	// test variants behaves differently, possibly due to lack of support for
-	// native overlays.
-	testenv.NeedsGo1Point(t, 16)
 	const src = `
 -- go.mod --
 module mod.test
@@ -1275,8 +1248,8 @@ func (Server) Foo() {}
 		// as invalid. So we need to wait for the metadata of main_test.go to be
 		// updated before moving other_test.go back to the main_test package.
 		env.Await(
-			env.DiagnosticAtRegexpWithMessage("other_test.go", "Server", "undeclared"),
-			env.DiagnosticAtRegexpWithMessage("main_test.go", "otherConst", "undeclared"),
+			env.DiagnosticAtRegexp("other_test.go", "Server"),
+			env.DiagnosticAtRegexp("main_test.go", "otherConst"),
 		)
 		env.RegexpReplace("other_test.go", "main", "main_test")
 		env.Await(
@@ -1333,7 +1306,7 @@ import (
 		env.Await(
 			OnceMet(
 				env.DoneWithOpen(),
-				env.DiagnosticAtRegexpWithMessage("a/main.go", "V", "declared but not used"),
+				env.DiagnosticAtRegexpWithMessage("a/main.go", "V", "not used"),
 			),
 		)
 		env.CloseBuffer("a/main.go")
@@ -1352,3 +1325,77 @@ import (
 		)
 	})
 }
+
+// Test that we don't get a version warning when the Go version in PATH is
+// supported.
+func TestOldGoNotification_SupportedVersion(t *testing.T) {
+	v := goVersion(t)
+	if v < lsp.OldestSupportedGoVersion() {
+		t.Skipf("go version 1.%d is unsupported", v)
+	}
+
+	Run(t, "", func(t *testing.T, env *Env) {
+		env.Await(
+			OnceMet(
+				InitialWorkspaceLoad,
+				NoShownMessage("upgrade"),
+			),
+		)
+	})
+}
+
+// Test that we do get a version warning when the Go version in PATH is
+// unsupported, though this test may never execute if we stop running CI at
+// legacy Go versions (see also TestOldGoNotification_Fake)
+func TestOldGoNotification_UnsupportedVersion(t *testing.T) {
+	v := goVersion(t)
+	if v >= lsp.OldestSupportedGoVersion() {
+		t.Skipf("go version 1.%d is supported", v)
+	}
+
+	Run(t, "", func(t *testing.T, env *Env) {
+		env.Await(
+			// Note: cannot use OnceMet(InitialWorkspaceLoad, ...) here, as the
+			// upgrade message may race with the IWL.
+			ShownMessage("Please upgrade"),
+		)
+	})
+}
+
+func TestOldGoNotification_Fake(t *testing.T) {
+	// Get the Go version from path, and make sure it's unsupported.
+	//
+	// In the future we'll stop running CI on legacy Go versions. By mutating the
+	// oldest supported Go version here, we can at least ensure that the
+	// ShowMessage pop-up works.
+	ctx := context.Background()
+	goversion, err := gocommand.GoVersion(ctx, gocommand.Invocation{}, &gocommand.Runner{})
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer func(t []lsp.GoVersionSupport) {
+		lsp.GoVersionTable = t
+	}(lsp.GoVersionTable)
+	lsp.GoVersionTable = []lsp.GoVersionSupport{
+		{GoVersion: goversion, InstallGoplsVersion: "v1.0.0"},
+	}
+
+	Run(t, "", func(t *testing.T, env *Env) {
+		env.Await(
+			// Note: cannot use OnceMet(InitialWorkspaceLoad, ...) here, as the
+			// upgrade message may race with the IWL.
+			ShownMessage("Please upgrade"),
+		)
+	})
+}
+
+// goVersion returns the version of the Go command in PATH.
+func goVersion(t *testing.T) int {
+	t.Helper()
+	ctx := context.Background()
+	goversion, err := gocommand.GoVersion(ctx, gocommand.Invocation{}, &gocommand.Runner{})
+	if err != nil {
+		t.Fatal(err)
+	}
+	return goversion
+}
